@@ -1,5 +1,5 @@
 void ctfail(void);
-void ctlogpn(char*, int, char*, ...);
+void ctlogpn(char*, int, char*, ...) __attribute__((format(printf, 3, 4)));
 #define ctlog(...) ctlogpn(__FILE__, __LINE__, __VA_ARGS__)
 #define assert(x) do if (!(x)) {\
 	ctlog("%s", "test: " #x);\
