@@ -340,6 +340,19 @@ cttestunderscore()
 }
 
 
+void
+cttest2cmdpacket()
+{
+    port = SERVER();
+    fd = mustdiallocal(port);
+    mustsend(fd, "use a\r\nuse b\r\n");
+    ckresp(fd, "USING a\r\n");
+    ckresp(fd, "USING b\r\n");
+
+    killsrv();
+}
+
+
 void
 cttesttoobig()
 {
